/**
 * Loading screen for multiply portals and error page. Views handling module.
 * This screen decorates the loading process for portals where loading delay is set (for any reason).
 *
 * @author Fedotov Dmytro <bas.jsdev@gmail.com>
 * @author Yaroslav Surilov <y.surilov@infomir.com>
 */

'use strict';

app.view.delay = (function () {
	var module = {},
		page, imageList, correctPortalURL, loadDelayTimer;


	// page initialization
	module.init = function () {
		page = new CPage();
		page.name = 'pageLoadDelay';
		page.Init(document.getElementById('pageLoadDelay'));

		page.EventHandler = function ( event ) {
			switch ( event.code ) {
				case KEYS.OK:
					// stop fast load page timeout and show (or load) another page
					clearTimeout(loadDelayTimer);
					// hide current and load new page
					page.Show(false);
					app.loadPortal(correctPortalURL);
					break;
				case KEYS.POWER:
					app.view.standByMode();
					break;
				case KEYS.MENU:
					if ( RULES['portalsLoader/useExtPortalsPage'] !== false ) {
						// stop fast load page timeout and show main page
						clearTimeout(loadDelayTimer);
						page.Show(false);
						// go to portal select page
						app.view.portals.show();
					}
					break;
			}
		};

		// apply localization
		document.body.querySelector('#pageLoadDelay .title').innerHTML = _('Loading portal...');
		document.body.querySelector('#pageLoadDelay .second').innerHTML = RULES['portalsLoader/useExtPortalsPage'] ? _('Press MENU to load portal selection screen') : '';

		imageList = ['bg2.jpg', 'loading.png'].map(function ( image ) {
			return PATH_IMG_SYSTEM + 'loader/' + image;
		});
	};


	/**
	 * Fill and show fast loader page
	 */
	module.show = function ( callback ) {
		if ( !page ) {
			module.init();
		}

		imageLoader(imageList, function () {
				echo('images has been loaded', 'imageLoader');
				// get right portal url
				if ( app.data.portalsData.enable ) {
					// multiportal mode: we should use default portal
					if ( app.data.portalsData.def ) {
						correctPortalURL = app.data.portalsData.portals[app.data.portalsData.def - 1].url
					} else {
						correctPortalURL = PATH_ROOT + 'services.html';
					}
				} else {
					// classic mode: we should check only portal1 and portal2
					correctPortalURL = app.data.environmentData.portal1 || app.data.environmentData.portal2;
				}
				// DHCP mode: we should use dhcp portal (highest priority)
				if ( app.data.environmentData.usePortalDHCP && app.data.environmentData.portalDHCP ) {
					correctPortalURL = app.data.environmentData.portalDHCP;
				}

				page.Show(true);

				// set timeout for default fast loading condition
				if ( accessControl.state && accessControl.data.events.loader ) {
					setTimeout(function () {
						accessControl.showLoginForm(function () {
							loadDelayTimer = setTimeout(function () {
								page.Show(false);
								callback();
							}, app.data.environmentData.portalLoadDelay * 1000);
						}, null, true);
					}, 0);
				} else {
					loadDelayTimer = setTimeout(function () {
						page.Show(false);
						callback();
					}, app.data.environmentData.portalLoadDelay * 1000);
				}
			}
		);
	};


	// export
	return module;
})();
